import emails

from quixote import get_request

import storage

from form import *

widget_classes = {
    'string': StringWidget,
    'email': EmailWidget,
    'text': TextWidget,
    'bool': CheckboxWidget,
    'item': SingleSelectWidget,
    'file': FileWidget,
}

widget_extra_attributes = {
    'text': ('cols', 'rows'),
    'string': ('size',),
}


class Consultation(storage.Storable):
    key = 'id'
    names = 'consultations'
    receiver = None
    roles = None
    category = None

    def __init__(self):
        self.id = ''
        self.name = ''
        self.questions = []
        self.receiver = ''
        self.category = ''
        self.roles = []

    def create_form(self, page_no, answers = {}):
        form = Form(enctype = "multipart/form-data")
        current_page = 0
        for i, question in enumerate(self.questions):
            if question['answer'] == 'newpage':
                current_page += 1
                if current_page > page_no:
                    break
                continue
            if current_page != page_no:
                continue
            if question['answer'] == 'title':
                form.widgets.append(HtmlWidget(htmltext("<h3>%s</h3>" % question['question'])))
                continue
            if question['answer'] == 'subtitle':
                form.widgets.append(HtmlWidget(htmltext("<h4>%s</h4>" % question['question'])))
                continue
            if question['answer'] == 'comment':
                form.widgets.append(HtmlWidget(htmltext("<p>%s</p>" % question['question'])))
                continue

            widget_class = widget_classes[question['answer']]

            kwargs = {'required': question.get('required', True)}
            if question['answer'] == 'item':
                kwargs['options'] = question.get('items', ['---'])
                if question.get('show-as-radio'):
                    widget_class = RadiobuttonsWidget
            value = None
            if answers.has_key(question['question']):
                kwargs['value'] = answers[question['question']]
            if widget_extra_attributes.has_key(question['answer']):
                for k in widget_extra_attributes[question['answer']]:
                    v = question.get(k, None)
                    if v is not None:
                        kwargs[k] = v
            form.add(widget_class, "f%d" % i, title = question['question'], **kwargs)

        if page_no != 0:
            form.add_submit("previous", _("Previous"))
        form.add_submit("submit", _("Next"))

        return form

    def get_data(self, form):
        d = {}
        for i, question in enumerate(self.questions):
            if question['answer'] in ('title', 'subtitle', 'comment'):
                continue
            if form.get_widget('f%d' % i):
                d[question['question']] = form.get_widget('f%d' % i).parse()
        return d

