import emails

from quixote import get_request, get_session

import storage

from form import *

widget_classes = {
    'string': StringWidget,
    'email': EmailWidget,
    'text': TextWidget,
    'bool': CheckboxWidget,
    'item': SingleSelectWidget,
    'file': FakeFileWidget,
}

widget_extra_attributes = {
    'text': ('cols', 'rows'),
    'string': ('size',),
}


class FormDef(storage.Storable):
    key = 'id'
    names = 'formdefs'
    receiver = None
    roles = None
    category = None

    def __init__(self):
        self.id = ''
        self.name = ''
        self.fields = []
        self.receiver = ''
        self.category = ''
        self.roles = []

    def create_form(self):  # XXX: merge create_form and create_view_form
        form = Form(enctype = "multipart/form-data", use_tokens = False)
        for i, field in enumerate(self.fields):
            if field['type'] == 'title':
                form.widgets.append(HtmlWidget(htmltext("<h3>%s</h3>" % field['name'])))
                continue
            if field['type'] == 'subtitle':
                form.widgets.append(HtmlWidget(htmltext("<h4>%s</h4>" % field['name'])))
                continue
            if field['type'] == 'comment':
                form.widgets.append(HtmlWidget(htmltext("<p>%s</p>" % field['name'])))
                continue
            
            widget_class = widget_classes[field['type']]

            kwargs = {'required': field.get('required', True)}
            if field['type'] == 'item':
                kwargs['options'] = field.get('items', ['---'])
                if field.get('show-as-radio'):
                    widget_class = RadiobuttonsWidget
            if widget_extra_attributes.has_key(field['type']):
                for k in widget_extra_attributes[field['type']]:
                    v = field.get(k, None)
                    if v is not None:
                        kwargs[k] = v
            form.add(widget_class, "f%d" % i, title = field['name'], **kwargs)
        return form

    def create_view_form(self, dict = {}):
        form = Form(enctype = "multipart/form-data")
        session = get_session()
        for i, field in enumerate(self.fields):
            kwargs = {}
            if field['type'] == 'title':
                form.widgets.append(HtmlWidget(htmltext("<h3>%s</h3>" % field['name'])))
                continue
            if field['type'] == 'subtitle':
                form.widgets.append(HtmlWidget(htmltext("<h4>%s</h4>" % field['name'])))
                continue
            if field['type'] == 'comment':
                form.widgets.append(HtmlWidget(htmltext("<p>%s</p>" % field['name'])))
                continue

            widget_class = widget_classes[field['type']]
            value = dict.get(field['name'], '')
            if field['type'] == 'item':
                widget_class = StringWidget

            if field['type'] == 'file':
                kwargs['preview'] = True
            if field['type'] == 'bool':
                kwargs['disabled'] = 'disabled'

            if widget_extra_attributes.has_key(field['type']):
                for k in widget_extra_attributes[field['type']]:
                    v = field.get(k, None)
                    if v is not None:
                        kwargs[k] = v

            if field['type'] == 'file':
                widget_class = FakeFileWidget

            form.add(widget_class, "f%d" % i, title = field['name'],
                    readonly = "readonly", value = value, **kwargs)
        return form

    def get_data(self, form):
        d = {}
        for i, field in enumerate(self.fields):
            if field['type'] in ('title', 'subtitle', 'comment'):
                continue
            d[field['name']] = form.get_widget('f%d' % i).parse()
        return d

    def notify_new(self, formdata):
        receiver = storage.get_storage().retrieve('roles', self.receiver)
        if not receiver.emails:
            return
        url = '%s/%s/status' % (get_request().get_url(1), formdata.id)
        mail_body = _("""Hi,

A new form has been submitted on the website, you can consult it with this
link: %(url)s
""") % {'url': url}
        emails.email(_("A new form has been submitted"), mail_body, receiver.emails[0],
                bcc = receiver.emails[1:])

    def notify_change(self, formdata):
        if not formdata.user_id:
            return
        try:
            user = storage.get_storage().retrieve('users', formdata.user_id)
        except KeyError:
            return
        if not user.email:
            return

        receiver = storage.get_storage().retrieve('roles', self.receiver)
        if receiver.emails:
            from_email = receiver.emails[0]
        else:
            from_email = None
        
        url = '%s/' % get_request().get_url(1)
        mail_body = _("""Hi,

Status of the form you submitted just changed, you can consult it with this
link: %(url)s
""") % {'url': url}
        emails.email(_('Form status change'), mail_body, user.email,
                replyto = from_email)


