from quixote.session import Session, SessionManager
import storage

class BasicSession(Session, storage.Storable):
    key = 'id'
    names = 'sessions'

    name_identifier = None
    after_url = None
    anonymous_id = None
    lasso_session_dump = None
    lasso_anonymous_identity_dump = None
    tempfiles = None

    def has_info(self):
        return self.name_identifier or self.after_url or self.anonymous_id or \
            self.tempfiles or self.lasso_session_dump or \
            self.lasso_anonymous_identity_dump or Session.has_info(self)
    is_dirty = has_info


class PickleSessionManager(SessionManager):
    def __init__(self):
        self.storage = storage.get_storage()
        SessionManager.__init__(self, session_class=BasicSession)

    def forget_changes(self, session):
        pass

    def __getitem__(self, session_id):
        try:
            return storage.get_storage().retrieve('sessions', session_id)
        except KeyError:
            raise KeyError

    def get(self, session_id, default = None):
        try:
            return storage.get_storage().retrieve('sessions', session_id)
        except KeyError:
            return default

    def commit_changes(self, session):
        if session and session.id:
            storage.get_storage().store(session)
        
    def keys(self):
        return storage.get_storage().keys('sessions')

    def values(self):
        return [self.get(k) for k in self.keys()]

    def items(self):
        return [(k, self.get(k)) for k in self.keys()]

    def has_key(self, session_id):
        return session_id in self.keys()

    def __setitem__(self, session_id, session):
        storage.get_storage().store(session)

    def __delitem__(self, session_id):
        if not session_id:
            return
        try:
            storage.get_storage().remove_id('sessions', session_id)
        except OSError:
            raise KeyError
